import mocapx
import mocapx.commands as mcpx_cmds
from mocapx.ui.about_dialog import AboutDialog
from mocapx.ui.poselib_editor import create_poselib_editor
from mocapx.lib.utils import get_mobject, adapt_name, import_scene
from mocapx.lib.nodes import get_selected_attrs, select_node, get_adapter_by_source,\
    get_all_active_data_sources, get_realtime_device_live, set_realtime_device_live,\
    get_realtime_device_conn_status

import maya.cmds as mc
import maya.api.OpenMaya as OpenMaya


def _null():
    pass


def create_shelf():
    shelf = MocapxShelf(name="MocapX")
    return shelf


class SimpleButton(object):
    def __init__(self, command=_null, icon="commandButton.png", annotation_text=""):
        self._callback = _null
        self.button_name = mc.shelfButton(
            width=32, height=32, image=icon, command=command, annotation=annotation_text)

    def set_command(self, command):
        mc.shelfButton(self.button_name, edit=True, command=command)


class DynamicButton(SimpleButton):
    def __init__(self, command=_null, icon="commandButton.png", annotation_text=""):
        super(DynamicButton, self).__init__(command=command, icon=icon, annotation_text=annotation_text)
        self._menu_codes = {'left': 1, 'middle': 2}
        self._menu_names = {'left': None, 'middle': None}
        self.node_add_cbid = None
        self.node_remove_cbid = None
        self.rename_callbacks = dict()
        self._callback = _null
        self._nodes_type = None

    @staticmethod
    def bound_callback(callback, node_name=None):
        bound_name = node_name

        def bound(*args, **kwargs):
            callback(bound_name)

        def unbound(*args, **kwargs):
            callback()

        if bound_name:
            return bound
        else:
            return unbound

    @staticmethod
    def _create_menu(parent, payload):
        for label, command in payload:
            mc.menuItem(p=parent, label=label, c=command)

    def create_menu(self, button, payload):
        if button not in self._menu_names.keys():
            raise ValueError('Cannot create menu for button {}'.format(button))
        if self._menu_names[button]:
            mc.deleteUI(self._menu_names[button])
        self._menu_names[button] = mc.popupMenu(
            parent=self.button_name, button=self._menu_codes[button])
        DynamicButton._create_menu(self._menu_names[button], payload)

    def get_callback(self, node_name=None):
        return DynamicButton.bound_callback(self._callback, node_name)

    def delete_menu(self, button):
        if button not in self._menu_names.keys():
            raise ValueError('Cannot create menu for button {}'.format(button))
        if self._menu_names[button]:
            mc.deleteUI(self._menu_names[button])
            self._menu_names[button] = None

    def update_menus(self, nodes=None):
        if nodes is None:
            nodes = map(adapt_name, mc.ls(type=self._nodes_type))
        if not nodes:
            self.set_command(self.get_callback())
            self.delete_menu('left')
        elif len(nodes) == 1:
            self.set_command(self.get_callback(nodes[0]))
            self.delete_menu('left')
        else:
            self.set_command(_null)
            payload = [("Add to {}".format(node), self.get_callback(node))
                       for node in nodes]
            self.create_menu('left', payload)

    def node_add_cb(self, node, clientData=None):
        if not mocapx.suppress_on_scene_loading:
            node_name = OpenMaya.MFnDependencyNode(node).name()

            cd_id = OpenMaya.MNodeMessage.addNameChangedCallback(
                node, self.node_rename_cb)
            self.rename_callbacks[node_name] = cd_id
            self.update_menus()

    def node_remove_cb(self, node, clientData=None):
        if not mocapx.suppress_on_scene_loading:
            node_list = mc.ls(type=self._nodes_type)
            node_name = OpenMaya.MFnDependencyNode(node).name()

            cb_id = self.rename_callbacks[node_name]
            OpenMaya.MMessage.removeCallback(cb_id)
            del self.rename_callbacks[node_name]
            node_list.remove(node_name)
            self.update_menus(node_list)

    def node_rename_cb(self, node, prevName, clientData=None):
        if not mocapx.suppress_on_scene_loading:
            new_name = OpenMaya.MFnDependencyNode(node).name()
            new_name = adapt_name(new_name)
            self.rename_callbacks[new_name] = self.rename_callbacks.pop(prevName)
            self.update_menus()

    def add_callbacks(self):
        node_list = mc.ls(type=self._nodes_type)
        for node_name in node_list:
            node_mobject = get_mobject(node_name)
            if node_mobject:
                cd_id = OpenMaya.MNodeMessage.addNameChangedCallback(
                    node_mobject, self.node_rename_cb)
                self.rename_callbacks[node_name] = cd_id

        self.node_add_cbid = OpenMaya.MDGMessage.addNodeAddedCallback(
            self.node_add_cb, self._nodes_type)

        self.node_remove_cbid = OpenMaya.MDGMessage.addNodeRemovedCallback(
            self.node_remove_cb, self._nodes_type)

    def refresh_callbacks(self):
        if self.rename_callbacks:
            for cb_id in self.rename_callbacks.values():
                OpenMaya.MMessage.removeCallback(cb_id)
            self.rename_callbacks = dict()
        if self.node_add_cbid:
            OpenMaya.MMessage.removeCallback(self.node_add_cbid)
            self.node_add_cbid = None
        if self.node_remove_cbid:
            OpenMaya.MMessage.removeCallback(self.node_remove_cbid)
            self.node_remove_cbid = None

        self.update_menus()
        self.add_callbacks()


class EditorButton(SimpleButton):
    @staticmethod
    def mcpx_shelf_editor():
        create_poselib_editor()

    def __init__(self):
        super(EditorButton, self).__init__(
            command=EditorButton.mcpx_shelf_editor,
            icon="MCPXEditor.png",
            annotation_text="Show MocapX Poselib Editor")


class RealtimeDeviceButton(SimpleButton):
    @staticmethod
    def action_handler():
        selection = mc.ls(sl=True)
        if len(selection):
            if mc.objectType(selection[0]) == 'MCPXAdapter':
                node = mcpx_cmds.create_realtime_device(adapter_name=selection[0])
            else:
                adapter = get_adapter_by_source(selection[0])
                node = mcpx_cmds.create_realtime_device(adapter_name=adapter)
        else:
            node = mcpx_cmds.create_realtime_device()
        select_node(node)

    def __init__(self):
        super(RealtimeDeviceButton, self).__init__(
            command=RealtimeDeviceButton.action_handler,
            icon="MCPXRealTimeDeviceCreate.png",
            annotation_text="Create MocapX Realtime Device")


class ClipreaderButton(SimpleButton):
    @staticmethod
    def action_handler():
        selection = mc.ls(sl=True)
        if len(selection):
            if mc.objectType(selection[0]) == 'MCPXAdapter':
                node = mcpx_cmds.create_clipreader(adapter_name=selection[0])
            else:
                adapter = get_adapter_by_source(selection[0])
                node = mcpx_cmds.create_clipreader(adapter_name=adapter)
        else:
            node = mcpx_cmds.create_clipreader()
        select_node(node)

    def __init__(self):
        super(ClipreaderButton, self).__init__(
            command=ClipreaderButton.action_handler,
            icon="MCPXClipReaderCreate.png",
            annotation_text="Create MocapX Clipreader")


class PoselibButton(DynamicButton):
    @staticmethod
    def mcpx_shelf_poselib(poselib=None):
        selection = mc.ls(sl=True)
        if not poselib:
            poselib = mcpx_cmds.create_empty_poselib()
            mc.select(selection, r=True)

        if selection:
            mcpx_cmds.add_pose_to_poselib(poselib, selected_poses=True)

    def __init__(self):
        super(PoselibButton, self).__init__(
            command=_null,
            icon="MCPXCreatePoselib.png",
            annotation_text="Create MocapX Poselib")
        self._callback = PoselibButton.mcpx_shelf_poselib
        self._nodes_type = 'MCPXPoselib'

        self.create_menu(
            'middle', [("Create new Poselib", self.get_callback())])

        self.update_menus()
        self.add_callbacks()


class AttrCollButton(DynamicButton):
    @staticmethod
    def mcpx_shelf_attr_collection(collection=None):
        selection = mc.ls(sl=True)
        if not collection:
            collection = mcpx_cmds.create_empty_collection()
            mc.select(selection, r=True)

        if selection:
            selected_attrs = get_selected_attrs()
            if selected_attrs:
                mcpx_cmds.add_plugs_to_collection(
                    collection, selected_plugs=True)
            else:
                mcpx_cmds.add_controls_to_collection(
                    collection, selected_controls=True)

    def __init__(self):
        super(AttrCollButton, self).__init__(
            command=_null,
            icon="MCPXCreateCollection.png",
            annotation_text="Create MocapX Attribute Collection")
        self._callback = AttrCollButton.mcpx_shelf_attr_collection
        self._nodes_type = 'MCPXAttributeCollection'

        self.create_menu(
            'middle', [("Create new Attribute Collection", self.get_callback())])

        self.update_menus()
        self.add_callbacks()


class PoseButton(DynamicButton):
    @staticmethod
    def mcpx_shelf_pose(poselib=None):
        selection = mc.ls(sl=True)
        if selection:
            pose = mcpx_cmds.create_empty_pose(pose_name=None)
            mc.select(selection, r=True)
            selected_attrs = get_selected_attrs()
            if selected_attrs:
                # add plugs to empty pose
                mcpx_cmds.update_plugs_in_pose(pose, selected_plugs=True)
            else:
                # add controls to empty pose
                mcpx_cmds.update_controls_in_pose(pose, selected_controls=True)
            mcpx_cmds.add_pose_to_poselib(poselib, [pose])

    def __init__(self):
        super(PoseButton, self).__init__(
            command=_null,
            icon="MCPXCreatePose.png",
            annotation_text="Create MocapX Pose")
        self._callback = PoseButton.mcpx_shelf_pose
        self._nodes_type = 'MCPXPoselib'

        self.update_menus()
        self.add_callbacks()


class PlayAllButton(SimpleButton):
    @staticmethod
    def play_all():
        for device in get_all_active_data_sources("MCPXRealTimeDevice"):
            if get_realtime_device_conn_status(device) in (2, 3) and not get_realtime_device_live(device):
                set_realtime_device_live(device, True)

    def __init__(self):
        super(PlayAllButton, self).__init__(
            command=PlayAllButton.play_all,
            icon="MCPXPlayAll.png",
            annotation_text="Play all active Realtime Devices")


class PauseAllButton(SimpleButton):
    @staticmethod
    def pause_all():
        for device in get_all_active_data_sources("MCPXRealTimeDevice"):
            if get_realtime_device_conn_status(device) in (2, 3) and get_realtime_device_live(device):
                set_realtime_device_live(device, False)

    def __init__(self):
        super(PauseAllButton, self).__init__(
            command=PauseAllButton.pause_all,
            icon="MCPXPauseAll.png",
            annotation_text="Pause all running Realtime Devices")


class HeadCreate(SimpleButton):
    @staticmethod
    def create_head():
        import_scene(r'$MOCAPX_EXAMPLES/head_scene.ma', namespace='Natalie')

    def __init__(self):
        super(HeadCreate, self).__init__(
            command=HeadCreate.create_head,
            icon="MCPXHeadCreate.png",
            annotation_text="Get Natalie head")


class AboutButton(SimpleButton):
    @staticmethod
    def show_info():
        wgt = AboutDialog()
        wgt.show()

    def __init__(self):
        super(AboutButton, self).__init__(
            command=AboutButton.show_info,
            icon="MCPXInfo.png",
            annotation_text="About MocapX")


class BakeButton(SimpleButton):
    @staticmethod
    def run_bake():
        mcpx_cmds.bake_channels()

    def __init__(self):
        super(BakeButton, self).__init__(
            command=BakeButton.run_bake,
            icon='MCPXBake.png',
            annotation_text="About MocapX")


class MocapxShelf(object):
    def __init__(self, name):
        self._buttons = dict()
        self.name = name
        self._clean_old_shelf()
        mc.setParent(self.name)
        self.build()

    def add_button(self, label, button):
        self._buttons[label] = button
        return button

    def _clean_old_shelf(self):
        if mc.shelfLayout(self.name, ex=True):
            for each in mc.shelfLayout(self.name, q=True, ca=True) or []:
                mc.deleteUI(each)
        else:
            mc.shelfLayout(self.name, p="ShelfLayout")

    def build(self):
        self.add_button('editor', EditorButton())
        self.add_button('attr_collect', AttrCollButton())
        self.add_button('poselib', PoselibButton())
        self.add_button('pose', PoseButton())
        self.add_button('realtime_device', RealtimeDeviceButton())
        self.add_button('clipreader', ClipreaderButton())
        self.add_button('play_all', PlayAllButton())
        self.add_button('pause_all', PauseAllButton())
        self.add_button('bake_channels', BakeButton())
        self.add_button('head_create', HeadCreate())
        self.add_button('show_info', AboutButton())
